"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _index = _interopRequireDefault(require("index"));

var _utils = _interopRequireDefault(require("utils"));

var _providers = _interopRequireDefault(require("./providers"));

var _querystring = _interopRequireDefault(require("querystring"));

var Event =
/*#__PURE__*/
function () {
  function Event() {
    var mcashWeb = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
    (0, _classCallCheck2["default"])(this, Event);
    if (!mcashWeb || !(mcashWeb instanceof _index["default"])) throw new Error('Expected instance of McashWeb');
    this.mcashWeb = mcashWeb;
    this.injectPromise = _utils["default"].promiseInjector(this);
  }

  (0, _createClass2["default"])(Event, [{
    key: "setServer",
    value: function setServer() {
      var _this = this;

      var eventServer = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var healthcheck = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'healthcheck';
      if (!eventServer) return this.mcashWeb.eventServer = false;
      if (_utils["default"].isString(eventServer)) eventServer = new _providers["default"].HttpProvider(eventServer);
      if (!this.mcashWeb.isValidProvider(eventServer)) throw new Error('Invalid event server provided');
      this.mcashWeb.eventServer = eventServer;

      this.mcashWeb.eventServer.isConnected = function () {
        return _this.mcashWeb.eventServer.request(healthcheck).then(function () {
          return true;
        })["catch"](function () {
          return false;
        });
      };
    }
  }, {
    key: "getEventsByContractAddress",
    value: function getEventsByContractAddress() {
      var contractAddress = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var callback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

      var _Object$assign = Object.assign({
        sinceTimestamp: 0,
        eventName: false,
        blockNumber: false,
        size: 20,
        page: 1
      }, options),
          sinceTimestamp = _Object$assign.sinceTimestamp,
          since = _Object$assign.since,
          fromTimestamp = _Object$assign.fromTimestamp,
          eventName = _Object$assign.eventName,
          blockNumber = _Object$assign.blockNumber,
          size = _Object$assign.size,
          page = _Object$assign.page,
          onlyConfirmed = _Object$assign.onlyConfirmed,
          onlyUnconfirmed = _Object$assign.onlyUnconfirmed,
          previousLastEventFingerprint = _Object$assign.previousLastEventFingerprint,
          previousFingerprint = _Object$assign.previousFingerprint,
          fingerprint = _Object$assign.fingerprint,
          rawResponse = _Object$assign.rawResponse,
          sort = _Object$assign.sort,
          filters = _Object$assign.filters;

      if (!callback) return this.injectPromise(this.getEventsByContractAddress, contractAddress, options);
      fromTimestamp = fromTimestamp || sinceTimestamp || since;
      if (!this.mcashWeb.eventServer) return callback('No event server configured');
      var routeParams = [];
      if (!this.mcashWeb.isAddress(contractAddress)) return callback('Invalid contract address provided');
      if (eventName && !contractAddress) return callback('Usage of event name filtering requires a contract address');
      if (typeof fromTimestamp !== 'undefined' && !_utils["default"].isInteger(fromTimestamp)) return callback('Invalid fromTimestamp provided');
      if (!_utils["default"].isInteger(size)) return callback('Invalid size provided');

      if (size > 200) {
        console.warn('Defaulting to maximum accepted size: 200');
        size = 200;
      }

      if (!_utils["default"].isInteger(page)) return callback('Invalid page provided');
      if (blockNumber && !eventName) return callback('Usage of block number filtering requires an event name');
      if (contractAddress) routeParams.push(this.mcashWeb.address.fromHex(contractAddress));
      if (eventName) routeParams.push(eventName);
      if (blockNumber) routeParams.push(blockNumber);
      var qs = {
        size: size,
        page: page
      };

      if ((0, _typeof2["default"])(filters) === 'object' && Object.keys(filters).length > 0) {
        qs.filters = JSON.stringify(filters);
      }

      if (fromTimestamp) {
        qs.fromTimestamp = qs.since = fromTimestamp;
      }

      if (onlyConfirmed) qs.onlyConfirmed = onlyConfirmed;
      if (onlyUnconfirmed && !onlyConfirmed) qs.onlyUnconfirmed = onlyUnconfirmed;
      if (sort) qs.sort = sort;
      fingerprint = fingerprint || previousFingerprint || previousLastEventFingerprint;
      if (fingerprint) qs.fingerprint = fingerprint;
      return this.mcashWeb.eventServer.request("event/contract/".concat(routeParams.join('/'), "?").concat(_querystring["default"].stringify(qs))).then(function () {
        var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
        if (!data) return callback('Unknown error occurred');
        if (!_utils["default"].isArray(data)) return callback(data);
        return callback(null, rawResponse === true ? data : data.map(function (event) {
          return _utils["default"].mapEvent(event);
        }));
      })["catch"](function (err) {
        return callback(err.response && err.response.data || err);
      });
    }
  }, {
    key: "getEventsByTransactionID",
    value: function getEventsByTransactionID() {
      var transactionID = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var callback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

      if (_utils["default"].isFunction(options)) {
        callback = options;
        options = {};
      }

      if (!callback) return this.injectPromise(this.getEventsByTransactionID, transactionID, options);
      if (!this.mcashWeb.eventServer) return callback('No event server configured');
      return this.mcashWeb.eventServer.request("event/transaction/".concat(transactionID)).then(function () {
        var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
        if (!data) return callback('Unknown error occurred');
        if (!_utils["default"].isArray(data)) return callback(data);
        return callback(null, options.rawResponse === true ? data : data.map(function (event) {
          return _utils["default"].mapEvent(event);
        }));
      })["catch"](function (err) {
        return callback(err.response && err.response.data || err);
      });
    }
  }]);
  return Event;
}();

exports["default"] = Event;
//# sourceMappingURL=event.js.map