"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _index = _interopRequireDefault(require("index"));

var _utils = _interopRequireDefault(require("utils"));

var _ethersUtils = require("utils/ethersUtils");

var _paramValidator = _interopRequireDefault(require("paramValidator"));

var _address = require("utils/address");

var self; //helpers

function toHex(value) {
  return self.mcashWeb.address.toHex(value);
}

function fromUtf8(value) {
  return self.mcashWeb.fromUtf8(value);
}

function resultManager(transaction, callback) {
  if (transaction.Error) return callback(transaction.Error);

  if (transaction.result && transaction.result.message) {
    return callback(self.mcashWeb.toUtf8(transaction.result.message));
  }

  return callback(null, transaction);
}

var TransactionBuilder =
/*#__PURE__*/
function () {
  function TransactionBuilder() {
    var mcashWeb = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
    (0, _classCallCheck2["default"])(this, TransactionBuilder);
    if (!mcashWeb || !mcashWeb instanceof _index["default"]) throw new Error('Expected instance of McashWeb');
    self = this;
    this.mcashWeb = mcashWeb;
    this.injectPromise = _utils["default"].promiseInjector(this);
    this.validator = new _paramValidator["default"](mcashWeb);
  }

  (0, _createClass2["default"])(TransactionBuilder, [{
    key: "sendMcash",
    value: function sendMcash() {
      var to = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var amount = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      var from = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

      if (_utils["default"].isFunction(from)) {
        callback = from;
        from = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.sendMcash, to, amount, from); // accept amounts passed as strings

      amount = parseInt(amount);
      if (this.validator.notValid([{
        name: 'recipient',
        type: 'address',
        value: to
      }, {
        name: 'origin',
        type: 'address',
        value: from
      }, {
        names: ['recipient', 'origin'],
        type: 'notEqual',
        msg: 'Cannot transfer MCASH to the same account'
      }, {
        name: 'amount',
        type: 'integer',
        gt: 0,
        value: amount
      }], callback)) return;
      this.mcashWeb.fullNode.request('wallet/createtransaction', {
        to_address: toHex(to),
        owner_address: toHex(from),
        amount: amount
      }, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
  }, {
    key: "sendToken",
    value: function sendToken() {
      var to = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var amount = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      var tokenId = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
      var from = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;

      if (_utils["default"].isFunction(from)) {
        callback = from;
        from = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.sendToken, to, amount, tokenId, from);
      if (_utils["default"].isString(amount)) amount = parseInt(amount);
      if (this.validator.notValid([{
        name: 'recipient',
        type: 'address',
        value: to
      }, {
        name: 'origin',
        type: 'address',
        value: from
      }, {
        names: ['recipient', 'origin'],
        type: 'notEqual',
        msg: 'Cannot transfer tokens to the same account'
      }, {
        name: 'amount',
        type: 'integer',
        gt: 0,
        value: amount
      }, {
        name: 'token Id',
        type: 'tokenId',
        value: tokenId
      }], callback)) return;
      this.mcashWeb.fullNode.request('wallet/transferasset', {
        to_address: toHex(to),
        owner_address: toHex(from),
        asset_id: tokenId,
        amount: amount
      }, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
  }, {
    key: "purchaseToken",
    value: function purchaseToken() {
      var issuerAddress = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var tokenId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var amount = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
      var buyer = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;

      if (_utils["default"].isFunction(buyer)) {
        callback = buyer;
        buyer = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.purchaseToken, issuerAddress, tokenId, amount, buyer);
      if (this.validator.notValid([{
        name: 'buyer',
        type: 'address',
        value: buyer
      }, {
        name: 'issuer',
        type: 'address',
        value: issuerAddress
      }, {
        names: ['buyer', 'issuer'],
        type: 'notEqual',
        msg: 'Cannot purchase tokens from same account'
      }, {
        name: 'amount',
        type: 'integer',
        gt: 0,
        value: amount
      }, {
        name: 'token ID',
        type: 'tokenId',
        value: tokenId
      }], callback)) return;
      this.mcashWeb.fullNode.request('wallet/participateassetissue', {
        to_address: toHex(issuerAddress),
        owner_address: toHex(buyer),
        asset_id: tokenId,
        amount: amount
      }, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
  }, {
    key: "freezeBalance",
    value: function freezeBalance() {
      var amount = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
      var duration = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 3;
      var resource = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : "BANDWIDTH";
      var address = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : this.mcashWeb.defaultAddress.hex;
      var receiverAddress = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : undefined;
      var callback = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : false;

      if (_utils["default"].isFunction(receiverAddress)) {
        callback = receiverAddress;
        receiverAddress = undefined;
      }

      if (_utils["default"].isFunction(address)) {
        callback = address;
        address = this.mcashWeb.defaultAddress.hex;
      }

      if (_utils["default"].isFunction(duration)) {
        callback = duration;
        duration = 3;
      }

      if (_utils["default"].isFunction(resource)) {
        callback = resource;
        resource = "BANDWIDTH";
      }

      if (!callback) return this.injectPromise(this.freezeBalance, amount, duration, resource, address, receiverAddress);
      if (this.validator.notValid([{
        name: 'origin',
        type: 'address',
        value: address
      }, {
        name: 'receiver',
        type: 'address',
        value: receiverAddress,
        optional: true
      }, {
        name: 'amount',
        type: 'integer',
        gt: 0,
        value: amount
      }, {
        name: 'duration',
        type: 'integer',
        gte: 0,
        value: duration
      }, {
        name: 'resource',
        type: 'resource',
        value: resource,
        msg: 'Invalid resource provided: Expected "BANDWIDTH" or "ENERGY'
      }], callback)) return;
      var data = {
        owner_address: toHex(address),
        frozen_balance: amount,
        frozen_duration: duration,
        resource: resource
      };

      if (_utils["default"].isNotNullOrUndefined(receiverAddress) && toHex(receiverAddress) !== toHex(address)) {
        data.receiver_address = toHex(receiverAddress);
      }

      this.mcashWeb.fullNode.request('wallet/freezebalance', data, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
  }, {
    key: "unfreezeBalance",
    value: function unfreezeBalance() {
      var resource = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : "BANDWIDTH";
      var address = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.mcashWeb.defaultAddress.hex;
      var receiverAddress = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : undefined;
      var callback = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

      if (_utils["default"].isFunction(receiverAddress)) {
        callback = receiverAddress;
        receiverAddress = undefined;
      }

      if (_utils["default"].isFunction(address)) {
        callback = address;
        address = this.mcashWeb.defaultAddress.hex;
      }

      if (_utils["default"].isFunction(resource)) {
        callback = resource;
        resource = "BANDWIDTH";
      }

      if (!callback) return this.injectPromise(this.unfreezeBalance, resource, address, receiverAddress);
      if (this.validator.notValid([{
        name: 'origin',
        type: 'address',
        value: address
      }, {
        name: 'receiver',
        type: 'address',
        value: receiverAddress,
        optional: true
      }, {
        name: 'resource',
        type: 'resource',
        value: resource,
        msg: 'Invalid resource provided: Expected "BANDWIDTH" or "ENERGY'
      }], callback)) return;
      var data = {
        owner_address: toHex(address),
        resource: resource
      };

      if (_utils["default"].isNotNullOrUndefined(receiverAddress) && toHex(receiverAddress) !== toHex(address)) {
        data.receiver_address = toHex(receiverAddress);
      }

      this.mcashWeb.fullNode.request('wallet/unfreezebalance', data, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
  }, {
    key: "unfreezeAsset",
    value: function unfreezeAsset() {
      var address = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

      if (_utils["default"].isFunction(address)) {
        callback = address;
        address = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.unfreezeAsset, address);
      if (this.validator.notValid([{
        name: 'owner address',
        type: 'address',
        value: address
      }], callback)) return;
      var data = {
        owner_address: toHex(address)
      };
      this.mcashWeb.fullNode.request('wallet/unfreezeasset', data, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
  }, {
    key: "stake",
    value: function stake() {
      var amount = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
      var stakeDuration = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 3;
      var address = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

      if (_utils["default"].isFunction(address)) {
        callback = address;
        address = this.mcashWeb.defaultAddress.hex;
      }

      if (_utils["default"].isFunction(stakeDuration)) {
        callback = stakeDuration;
        stakeDuration = 3;
      }

      if (!callback) return this.injectPromise(this.stake, amount, stakeDuration, address);
      if (this.validator.notValid([{
        name: 'origin',
        type: 'address',
        value: address
      }, {
        name: 'amount',
        type: 'integer',
        gt: 0,
        value: amount
      }, {
        name: 'duration',
        type: 'integer',
        gte: 0,
        value: stakeDuration
      }], callback)) return;
      var data = {
        owner_address: toHex(address),
        stake_amount: amount,
        stake_duration: stakeDuration
      };
      this.mcashWeb.fullNode.request('wallet/stake', data, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
  }, {
    key: "unstake",
    value: function unstake() {
      var address = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

      if (_utils["default"].isFunction(address)) {
        callback = address;
        address = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.unstake, address);
      if (this.validator.notValid([{
        name: 'origin',
        type: 'address',
        value: address
      }], callback)) return;
      var data = {
        owner_address: toHex(address)
      };
      this.mcashWeb.fullNode.request('wallet/unstake', data, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
  }, {
    key: "withdrawBlockRewards",
    value: function withdrawBlockRewards() {
      var address = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

      if (_utils["default"].isFunction(address)) {
        callback = address;
        address = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.withdrawBlockRewards, address);
      if (this.validator.notValid([{
        name: 'origin',
        type: 'address',
        value: address
      }], callback)) return;
      this.mcashWeb.fullNode.request('wallet/withdrawbalance', {
        owner_address: toHex(address)
      }, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
  }, {
    key: "createWitness",
    value: function createWitness() {
      var witnessAddress = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var ownerAddress = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.mcashWeb.defaultAddress.hex;
      var url = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
      var callback = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

      if (_utils["default"].isValidURL(ownerAddress)) {
        callback = url || false;
        url = ownerAddress;
        ownerAddress = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.createWitness, witnessAddress, ownerAddress, url);
      if (this.validator.notValid([{
        name: 'owner',
        type: 'address',
        value: ownerAddress
      }, {
        name: 'witness',
        type: 'address',
        value: witnessAddress
      }, {
        name: 'url',
        type: 'url',
        value: url,
        msg: 'Invalid url provided'
      }], callback)) return;
      this.mcashWeb.fullNode.request('wallet/createwitness', {
        owner_address: toHex(ownerAddress),
        witness_address: toHex(witnessAddress),
        url: fromUtf8(url)
      }, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
  }, {
    key: "createAccount",
    value: function createAccount() {
      var accountAddress = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var ownerAddress = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
      if (!callback) return this.injectPromise(this.createAccount, accountAddress, ownerAddress);
      if (this.validator.notValid([{
        name: 'ownerAddress',
        type: 'address',
        value: ownerAddress
      }, {
        name: 'accountAddress',
        type: 'address',
        value: accountAddress
      }], callback)) return;
      this.mcashWeb.fullNode.request('wallet/createaccount', {
        owner_address: toHex(ownerAddress),
        account_address: toHex(accountAddress)
      }, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
  }, {
    key: "vote",
    value: function vote() {
      var voteAddress = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var ownerAddress = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

      if (_utils["default"].isFunction(ownerAddress)) {
        callback = ownerAddress;
        ownerAddress = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.vote, voteAddress, ownerAddress);
      if (this.validator.notValid([{
        name: 'ownerAddress',
        type: 'address',
        value: ownerAddress
      }, {
        name: 'voteAddress',
        type: 'address',
        value: voteAddress
      }], callback)) return;
      this.mcashWeb.fullNode.request('wallet/votewitnessaccount', {
        owner_address: toHex(ownerAddress),
        vote_address: toHex(voteAddress)
      }, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
  }, {
    key: "createSmartContract",
    value: function createSmartContract() {
      var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var issuerAddress = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

      if (_utils["default"].isFunction(issuerAddress)) {
        callback = issuerAddress;
        issuerAddress = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.createSmartContract, options, issuerAddress);
      var feeLimit = options.feeLimit || 1000000000;
      var userFeePercentage = options.userFeePercentage;

      if (typeof userFeePercentage !== 'number' && !userFeePercentage) {
        userFeePercentage = 100;
      }

      var originEnergyLimit = options.originEnergyLimit || 10000000;
      var callValue = options.callValue || 0;
      var tokenValue = options.tokenValue;
      var tokenId = options.tokenId || options.token_id;
      var _options$abi = options.abi,
          abi = _options$abi === void 0 ? false : _options$abi,
          _options$bytecode = options.bytecode,
          bytecode = _options$bytecode === void 0 ? false : _options$bytecode,
          _options$parameters = options.parameters,
          parameters = _options$parameters === void 0 ? [] : _options$parameters,
          _options$name = options.name,
          name = _options$name === void 0 ? "" : _options$name;

      if (abi && _utils["default"].isString(abi)) {
        try {
          abi = JSON.parse(abi);
        } catch (_unused) {
          return callback('Invalid options.abi provided');
        }
      }

      if (!_utils["default"].isArray(abi)) return callback('Invalid options.abi provided');
      var payable = abi.some(function (func) {
        return func.type === 'constructor' && func.payable;
      });
      if (this.validator.notValid([{
        name: 'bytecode',
        type: 'hex',
        value: bytecode
      }, {
        name: 'feeLimit',
        type: 'integer',
        value: feeLimit,
        gt: 0,
        lte: 100000000000
      }, {
        name: 'callValue',
        type: 'integer',
        value: callValue,
        gte: 0
      }, {
        name: 'userFeePercentage',
        type: 'integer',
        value: userFeePercentage,
        gte: 0,
        lte: 100
      }, {
        name: 'originEnergyLimit',
        type: 'integer',
        value: originEnergyLimit,
        gte: 0,
        lte: 10000000
      }, {
        name: 'parameters',
        type: 'array',
        value: parameters
      }, {
        name: 'issuer',
        type: 'address',
        value: issuerAddress
      }, {
        name: 'tokenValue',
        type: 'integer',
        value: tokenValue,
        gte: 0,
        optional: true
      }, {
        name: 'tokenId',
        type: 'integer',
        value: tokenId,
        gte: 0,
        optional: true
      }], callback)) return;
      if (payable && callValue == 0 && tokenValue == 0) return callback('When contract is payable, options.callValue or options.tokenValue must be a positive integer');
      if (!payable && (callValue > 0 || tokenValue > 0)) return callback('When contract is not payable, options.callValue and options.tokenValue must be 0');
      var constructorParams = abi.find(function (it) {
        return it.type === 'constructor';
      });

      if (typeof constructorParams !== 'undefined' && constructorParams) {
        var abiCoder = new _ethersUtils.AbiCoder();
        var types = [];
        var values = [];
        constructorParams = constructorParams.inputs;
        if (parameters.length != constructorParams.length) return callback("constructor needs ".concat(constructorParams.length, " but ").concat(parameters.length, " provided"));

        for (var i = 0; i < parameters.length; i++) {
          var type = constructorParams[i].type;
          var value = parameters[i];
          if (!type || !_utils["default"].isString(type) || !type.length) return callback('Invalid parameter type provided: ' + type);
          if (type == 'address') value = toHex(value).replace(_address.ADDRESS_PREFIX_REGEX, '0x');
          types.push(type);
          values.push(value);
        }

        try {
          parameters = abiCoder.encode(types, values).replace(/^(0x)/, '');
        } catch (ex) {
          return callback(ex);
        }
      } else parameters = '';

      var args = {
        owner_address: toHex(issuerAddress),
        fee_limit: parseInt(feeLimit),
        call_value: parseInt(callValue),
        consume_user_resource_percent: userFeePercentage,
        origin_energy_limit: originEnergyLimit,
        abi: JSON.stringify(abi),
        bytecode: bytecode,
        parameter: parameters,
        name: name
      }; // tokenValue and tokenId can cause errors if provided when the trx10 proposal has not been approved yet. So we set them only if they are passed to the method.

      if (_utils["default"].isNotNullOrUndefined(tokenValue)) args.call_token_value = parseInt(tokenValue);
      if (_utils["default"].isNotNullOrUndefined(tokenId)) args.token_id = parseInt(tokenId);
      this.mcashWeb.fullNode.request('wallet/deploycontract', args, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
  }, {
    key: "triggerSmartContract",
    value: function triggerSmartContract() {
      for (var _len = arguments.length, params = new Array(_len), _key = 0; _key < _len; _key++) {
        params[_key] = arguments[_key];
      }

      if ((0, _typeof2["default"])(params[2]) !== 'object') {
        params[2] = {
          feeLimit: params[2],
          callValue: params[3]
        };
        params.splice(3, 1);
      }

      return this._triggerSmartContract.apply(this, params);
    }
  }, {
    key: "_triggerSmartContract",
    value: function _triggerSmartContract(contractAddress, functionSelector) {
      var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      var parameters = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];
      var issuerAddress = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : false;

      if (_utils["default"].isFunction(issuerAddress)) {
        callback = issuerAddress;
        issuerAddress = this.mcashWeb.defaultAddress.hex;
      }

      if (_utils["default"].isFunction(parameters)) {
        callback = parameters;
        parameters = [];
      }

      if (!callback) {
        return this.injectPromise(this._triggerSmartContract, contractAddress, functionSelector, options, parameters, issuerAddress);
      }

      var _Object$assign = Object.assign({
        callValue: 0,
        feeLimit: 1000000000
      }, options),
          tokenValue = _Object$assign.tokenValue,
          tokenId = _Object$assign.tokenId,
          callValue = _Object$assign.callValue,
          feeLimit = _Object$assign.feeLimit;

      if (this.validator.notValid([{
        name: 'feeLimit',
        type: 'integer',
        value: feeLimit,
        gt: 0,
        lte: 100000000000
      }, {
        name: 'callValue',
        type: 'integer',
        value: callValue,
        gte: 0
      }, {
        name: 'parameters',
        type: 'array',
        value: parameters
      }, {
        name: 'contract',
        type: 'address',
        value: contractAddress
      }, {
        name: 'issuer',
        type: 'address',
        value: issuerAddress
      }, {
        name: 'tokenValue',
        type: 'integer',
        value: tokenValue,
        gte: 0,
        optional: true
      }, {
        name: 'tokenId',
        type: 'integer',
        value: tokenId,
        gte: 0,
        optional: true
      }, {
        name: 'function selector',
        type: 'not-empty-string',
        value: functionSelector
      }], callback)) return;
      functionSelector = functionSelector.replace('/\s*/g', '');

      if (parameters.length) {
        var abiCoder = new _ethersUtils.AbiCoder();
        var types = [];
        var values = [];

        for (var i = 0; i < parameters.length; i++) {
          var _parameters$i = parameters[i],
              type = _parameters$i.type,
              value = _parameters$i.value;
          if (!type || !_utils["default"].isString(type) || !type.length) return callback('Invalid parameter type provided: ' + type);
          if (type === 'address') value = toHex(value).replace(_address.ADDRESS_PREFIX_REGEX, '0x');
          types.push(type);
          values.push(value);
        }

        try {
          // workaround for unsupported mcashToken type
          types = types.map(function (type) {
            if (/mcashToken/.test(type)) {
              type = type.replace(/mcashToken/, 'uint256');
            }

            return type;
          });
          parameters = abiCoder.encode(types, values).replace(/^(0x)/, '');
        } catch (ex) {
          return callback(ex);
        }
      } else parameters = '';

      var args = {
        contract_address: toHex(contractAddress),
        owner_address: toHex(issuerAddress),
        function_selector: functionSelector,
        fee_limit: parseInt(feeLimit),
        call_value: parseInt(callValue),
        parameter: parameters
      };
      if (_utils["default"].isNotNullOrUndefined(tokenValue)) args.call_token_value = parseInt(tokenValue);
      if (_utils["default"].isNotNullOrUndefined(tokenId)) args.token_id = parseInt(tokenId);
      this.mcashWeb.fullNode.request('wallet/triggersmartcontract', args, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
  }, {
    key: "createToken",
    value: function createToken() {
      var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var issuerAddress = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

      if (_utils["default"].isFunction(issuerAddress)) {
        callback = issuerAddress;
        issuerAddress = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.createToken, options, issuerAddress);
      var _options$name2 = options.name,
          name = _options$name2 === void 0 ? false : _options$name2,
          _options$abbreviation = options.abbreviation,
          abbreviation = _options$abbreviation === void 0 ? false : _options$abbreviation,
          _options$description = options.description,
          description = _options$description === void 0 ? false : _options$description,
          _options$url = options.url,
          url = _options$url === void 0 ? false : _options$url,
          _options$totalSupply = options.totalSupply,
          totalSupply = _options$totalSupply === void 0 ? 0 : _options$totalSupply,
          _options$mcashRatio = options.mcashRatio,
          mcashRatio = _options$mcashRatio === void 0 ? 1 : _options$mcashRatio,
          _options$tokenRatio = options.tokenRatio,
          tokenRatio = _options$tokenRatio === void 0 ? 1 : _options$tokenRatio,
          _options$saleStart = options.saleStart,
          saleStart = _options$saleStart === void 0 ? Date.now() : _options$saleStart,
          _options$saleEnd = options.saleEnd,
          saleEnd = _options$saleEnd === void 0 ? false : _options$saleEnd,
          _options$freeBandwidt = options.freeBandwidth,
          freeBandwidth = _options$freeBandwidt === void 0 ? 0 : _options$freeBandwidt,
          _options$freeBandwidt2 = options.freeBandwidthLimit,
          freeBandwidthLimit = _options$freeBandwidt2 === void 0 ? 0 : _options$freeBandwidt2,
          _options$frozenAmount = options.frozenAmount,
          frozenAmount = _options$frozenAmount === void 0 ? 0 : _options$frozenAmount,
          _options$frozenDurati = options.frozenDuration,
          frozenDuration = _options$frozenDurati === void 0 ? 0 : _options$frozenDurati,
          voteScore = options.voteScore,
          precision = options.precision;
      if (this.validator.notValid([{
        name: 'Supply amount',
        type: 'positive-integer',
        value: totalSupply
      }, {
        name: 'MCASH ratio',
        type: 'positive-integer',
        value: mcashRatio
      }, {
        name: 'Token ratio',
        type: 'positive-integer',
        value: tokenRatio
      }, {
        name: 'token abbreviation',
        type: 'not-empty-string',
        value: abbreviation
      }, {
        name: 'token name',
        type: 'not-empty-string',
        value: name
      }, {
        name: 'token description',
        type: 'not-empty-string',
        value: description
      }, {
        name: 'token url',
        type: 'url',
        value: url
      }, {
        name: 'issuer',
        type: 'address',
        value: issuerAddress
      }, {
        name: 'sale start timestamp',
        type: 'integer',
        value: saleStart,
        gte: Date.now()
      }, {
        name: 'sale end timestamp',
        type: 'integer',
        value: saleEnd,
        gt: saleStart
      }, {
        name: 'Free bandwidth amount',
        type: 'integer',
        value: freeBandwidth,
        gte: 0
      }, {
        name: 'Free bandwidth limit',
        type: 'integer',
        value: freeBandwidthLimit,
        gte: 0
      }, {
        name: 'Frozen supply',
        type: 'integer',
        value: frozenAmount,
        gte: 0
      }, {
        name: 'Frozen duration',
        type: 'integer',
        value: frozenDuration,
        gte: 0
      }], callback)) return;
      if (_utils["default"].isNotNullOrUndefined(voteScore) && (!_utils["default"].isInteger(voteScore) || voteScore <= 0)) return callback('voteScore must be a positive integer greater than 0');
      if (_utils["default"].isNotNullOrUndefined(precision) && (!_utils["default"].isInteger(precision) || precision < 0 || precision > 8)) return callback('precision must be a positive integer >= 0 and <= 8');
      var data = {
        owner_address: toHex(issuerAddress),
        name: fromUtf8(name),
        abbr: fromUtf8(abbreviation),
        description: fromUtf8(description),
        url: fromUtf8(url),
        total_supply: totalSupply,
        mcash_num: mcashRatio,
        num: tokenRatio,
        start_time: saleStart,
        end_time: saleEnd,
        free_asset_bandwidth_limit: freeBandwidth,
        public_free_asset_bandwidth_limit: freeBandwidthLimit,
        frozen_supply: {
          frozen_amount: frozenAmount,
          frozen_days: frozenDuration
        }
      };

      if (precision && !isNaN(parseInt(precision))) {
        data.precision = parseInt(precision);
      }

      if (voteScore && !isNaN(parseInt(voteScore))) {
        data.vote_score = parseInt(voteScore);
      }

      this.mcashWeb.fullNode.request('wallet/createassetissue', data, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
  }, {
    key: "updateAccount",
    value: function updateAccount() {
      var accountName = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var address = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

      if (_utils["default"].isFunction(address)) {
        callback = address;
        address = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) {
        return this.injectPromise(this.updateAccount, accountName, address);
      }

      if (this.validator.notValid([{
        name: 'Name',
        type: 'not-empty-string',
        value: accountName
      }, {
        name: 'origin',
        type: 'address',
        value: address
      }], callback)) return;
      this.mcashWeb.fullNode.request('wallet/updateaccount', {
        account_name: fromUtf8(accountName),
        owner_address: toHex(address)
      }, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
  }, {
    key: "updateToken",
    value: function updateToken() {
      var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var issuerAddress = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

      if (_utils["default"].isFunction(issuerAddress)) {
        callback = issuerAddress;
        issuerAddress = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.updateToken, options, issuerAddress);
      var _options$description2 = options.description,
          description = _options$description2 === void 0 ? false : _options$description2,
          _options$url2 = options.url,
          url = _options$url2 === void 0 ? false : _options$url2,
          _options$freeBandwidt3 = options.freeBandwidth,
          freeBandwidth = _options$freeBandwidt3 === void 0 ? 0 : _options$freeBandwidt3,
          _options$freeBandwidt4 = options.freeBandwidthLimit,
          freeBandwidthLimit = _options$freeBandwidt4 === void 0 ? 0 : _options$freeBandwidt4;
      if (this.validator.notValid([{
        name: 'token description',
        type: 'not-empty-string',
        value: description
      }, {
        name: 'token url',
        type: 'url',
        value: url
      }, {
        name: 'issuer',
        type: 'address',
        value: issuerAddress
      }, {
        name: 'Free bandwidth amount',
        type: 'positive-integer',
        value: freeBandwidth
      }, {
        name: 'Free bandwidth limit',
        type: 'positive-integer',
        value: freeBandwidthLimit
      }], callback)) return;
      this.mcashWeb.fullNode.request('wallet/updateasset', {
        owner_address: toHex(issuerAddress),
        description: fromUtf8(description),
        url: fromUtf8(url),
        new_limit: parseInt(freeBandwidth),
        new_public_limit: parseInt(freeBandwidthLimit)
      }, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
  }, {
    key: "sendAsset",
    value: function sendAsset() {
      return this.sendToken.apply(this, arguments);
    }
  }, {
    key: "purchaseAsset",
    value: function purchaseAsset() {
      return this.purchaseToken.apply(this, arguments);
    }
  }, {
    key: "createAsset",
    value: function createAsset() {
      return this.createToken.apply(this, arguments);
    }
  }, {
    key: "updateAsset",
    value: function updateAsset() {
      return this.updateToken.apply(this, arguments);
    }
    /**
     * Creates a proposal to modify the network.
     * Can only be created by a current Super Representative.
     */

  }, {
    key: "createProposal",
    value: function createProposal() {
      var parameters = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var issuerAddress = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

      if (_utils["default"].isFunction(issuerAddress)) {
        callback = issuerAddress;
        issuerAddress = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.createProposal, parameters, issuerAddress);
      if (this.validator.notValid([{
        name: 'issuer',
        type: 'address',
        value: issuerAddress
      }], callback)) return;
      var invalid = 'Invalid proposal parameters provided';
      if (!parameters) return callback(invalid);
      if (!_utils["default"].isArray(parameters)) parameters = [parameters];
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = parameters[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var parameter = _step.value;
          if (!_utils["default"].isObject(parameter)) return callback(invalid);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator["return"] != null) {
            _iterator["return"]();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      this.mcashWeb.fullNode.request('wallet/proposalcreate', {
        owner_address: toHex(issuerAddress),
        parameters: parameters
      }, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
    /**
     * Deletes a network modification proposal that the owner issued.
     * Only current Super Representative can vote on a proposal.
     */

  }, {
    key: "deleteProposal",
    value: function deleteProposal() {
      var proposalId = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var issuerAddress = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

      if (_utils["default"].isFunction(issuerAddress)) {
        callback = issuerAddress;
        issuerAddress = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.deleteProposal, proposalId, issuerAddress);
      if (this.validator.notValid([{
        name: 'issuer',
        type: 'address',
        value: issuerAddress
      }, {
        name: 'proposalId',
        type: 'integer',
        value: proposalId,
        gte: 0
      }], callback)) return;
      this.mcashWeb.fullNode.request('wallet/proposaldelete', {
        owner_address: toHex(issuerAddress),
        proposal_id: proposalId
      }, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
    /**
     * Adds a vote to an issued network modification proposal.
     * Only current Super Representative can vote on a proposal.
     */

  }, {
    key: "voteProposal",
    value: function voteProposal() {
      var proposalId = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var isApproval = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var voterAddress = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

      if (_utils["default"].isFunction(voterAddress)) {
        callback = voterAddress;
        voterAddress = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.voteProposal, proposalId, isApproval, voterAddress);
      if (this.validator.notValid([{
        name: 'voter',
        type: 'address',
        value: voterAddress
      }, {
        name: 'proposalId',
        type: 'integer',
        value: proposalId,
        gte: 0
      }, {
        name: 'has approval',
        type: 'boolean',
        value: isApproval
      }], callback)) return;
      this.mcashWeb.fullNode.request('wallet/proposalapprove', {
        owner_address: toHex(voterAddress),
        proposal_id: proposalId,
        is_add_approval: isApproval
      }, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
    /**
     * Create an exchange between a token and MCASH.
     * Token Name should be a CASE SENSITIVE string.
     */

  }, {
    key: "createMcashExchange",
    value: function createMcashExchange(tokenId, tokenBalance, mcashBalance) {
      var ownerAddress = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;

      if (_utils["default"].isFunction(ownerAddress)) {
        callback = ownerAddress;
        ownerAddress = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.createMcashExchange, tokenId, tokenBalance, mcashBalance, ownerAddress);
      if (this.validator.notValid([{
        name: 'owner',
        type: 'address',
        value: ownerAddress
      }, {
        name: 'token id',
        type: 'integer',
        value: tokenId,
        gte: 0
      }, {
        name: 'token balance',
        type: 'positive-integer',
        value: tokenBalance
      }, {
        name: 'mcash balance',
        type: 'positive-integer',
        value: mcashBalance
      }], callback)) return;
      this.mcashWeb.fullNode.request('wallet/exchangecreate', {
        owner_address: toHex(ownerAddress),
        first_token_id: tokenId,
        first_token_balance: tokenBalance,
        second_token_id: 0,
        // Constant for MCASH.
        second_token_balance: mcashBalance
      }, 'post').then(function (resources) {
        callback(null, resources);
      })["catch"](function (err) {
        return callback(err);
      });
    }
    /**
     * Create an exchange between a token and another token.
     * DO NOT USE THIS FOR MCASH.
     * Token Names should be a CASE SENSITIVE string.
     */

  }, {
    key: "createTokenExchange",
    value: function createTokenExchange(firstTokenId, firstTokenBalance, secondTokenId, secondTokenBalance) {
      var ownerAddress = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : false;

      if (_utils["default"].isFunction(ownerAddress)) {
        callback = ownerAddress;
        ownerAddress = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.createTokenExchange, firstTokenId, firstTokenBalance, secondTokenId, secondTokenBalance, ownerAddress);
      if (this.validator.notValid([{
        name: 'owner',
        type: 'address',
        value: ownerAddress
      }, {
        name: 'first token id',
        type: 'integer',
        value: firstTokenId,
        gte: 0
      }, {
        name: 'second token id',
        type: 'integer',
        value: secondTokenId,
        gte: 0
      }, {
        name: 'first token balance',
        type: 'positive-integer',
        value: firstTokenBalance
      }, {
        name: 'second token balance',
        type: 'positive-integer',
        value: secondTokenBalance
      }], callback)) return;
      this.mcashWeb.fullNode.request('wallet/exchangecreate', {
        owner_address: toHex(ownerAddress),
        first_token_id: firstTokenId,
        first_token_balance: firstTokenBalance,
        second_token_id: secondTokenId,
        second_token_balance: secondTokenBalance
      }, 'post').then(function (resources) {
        callback(null, resources);
      })["catch"](function (err) {
        return callback(err);
      });
    }
    /**
     * Adds tokens into a bancor style exchange.
     * Will add both tokens at market rate.
     * Use 0 for the constant value for MCASH.
     */

  }, {
    key: "injectExchangeTokens",
    value: function injectExchangeTokens() {
      var exchangeId = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var tokenId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var tokenAmount = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
      var ownerAddress = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;

      if (_utils["default"].isFunction(ownerAddress)) {
        callback = ownerAddress;
        ownerAddress = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.injectExchangeTokens, exchangeId, tokenId, tokenAmount, ownerAddress);
      if (this.validator.notValid([{
        name: 'owner',
        type: 'address',
        value: ownerAddress
      }, {
        name: 'tokenId',
        type: 'integer',
        value: tokenId,
        gte: 0
      }, {
        name: 'tokenAmount',
        type: 'integer',
        value: tokenAmount,
        gte: 1
      }, {
        name: 'exchangeId',
        type: 'integer',
        value: exchangeId,
        gte: 0
      }], callback)) return;
      this.mcashWeb.fullNode.request('wallet/exchangeinject', {
        owner_address: toHex(ownerAddress),
        exchange_id: exchangeId,
        token_id: tokenId,
        quant: tokenAmount
      }, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
    /**
     * Withdraws tokens from a bancor style exchange.
     * Will withdraw at market rate both tokens.
     * Use "_" for the constant value for TRX.
     */

  }, {
    key: "withdrawExchangeTokens",
    value: function withdrawExchangeTokens() {
      var exchangeId = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var tokenId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var tokenAmount = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
      var ownerAddress = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;

      if (_utils["default"].isFunction(ownerAddress)) {
        callback = ownerAddress;
        ownerAddress = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.withdrawExchangeTokens, exchangeId, tokenId, tokenAmount, ownerAddress);
      if (this.validator.notValid([{
        name: 'owner',
        type: 'address',
        value: ownerAddress
      }, {
        name: 'tokenId',
        type: 'integer',
        value: tokenId,
        gte: 0
      }, {
        name: 'tokenAmount',
        type: 'integer',
        value: tokenAmount,
        gte: 1
      }, {
        name: 'exchangeId',
        type: 'integer',
        value: exchangeId,
        gte: 0
      }], callback)) return;
      this.mcashWeb.fullNode.request('wallet/exchangewithdraw', {
        owner_address: toHex(ownerAddress),
        exchange_id: exchangeId,
        token_id: tokenId,
        quant: tokenAmount
      }, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
    /**
     * Trade tokens on a bancor style exchange.
     * Expected value is a validation and used to cap the total amt of token 2 spent.
     * Use "_" for the constant value for TRX.
     */

  }, {
    key: "tradeExchangeTokens",
    value: function tradeExchangeTokens() {
      var exchangeId = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var tokenId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var tokenAmountSold = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
      var tokenAmountExpected = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0;
      var ownerAddress = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : false;

      if (_utils["default"].isFunction(ownerAddress)) {
        callback = ownerAddress;
        ownerAddress = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.tradeExchangeTokens, exchangeId, tokenId, tokenAmountSold, tokenAmountExpected, ownerAddress);
      if (this.validator.notValid([{
        name: 'owner',
        type: 'address',
        value: ownerAddress
      }, {
        name: 'token name',
        type: 'integer',
        value: tokenId,
        gte: 0
      }, {
        name: 'tokenAmountSold',
        type: 'integer',
        value: tokenAmountSold,
        gte: 1
      }, {
        name: 'tokenAmountExpected',
        type: 'integer',
        value: tokenAmountExpected,
        gte: 1
      }, {
        name: 'exchangeId',
        type: 'integer',
        value: exchangeId,
        gte: 0
      }], callback)) return;
      this.mcashWeb.fullNode.request('wallet/exchangetransaction', {
        owner_address: toHex(ownerAddress),
        exchange_id: exchangeId,
        token_id: tokenId,
        quant: tokenAmountSold,
        expected: tokenAmountExpected
      }, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
    /**
     * Update userFeePercentage.
     */

  }, {
    key: "updateSetting",
    value: function updateSetting() {
      var contractAddress = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var userFeePercentage = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var ownerAddress = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

      if (_utils["default"].isFunction(ownerAddress)) {
        callback = ownerAddress;
        ownerAddress = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.updateSetting, contractAddress, userFeePercentage, ownerAddress);
      if (this.validator.notValid([{
        name: 'owner',
        type: 'address',
        value: ownerAddress
      }, {
        name: 'contract',
        type: 'address',
        value: contractAddress
      }, {
        name: 'userFeePercentage',
        type: 'integer',
        value: userFeePercentage,
        gte: 0,
        lte: 100
      }], callback)) return;
      this.mcashWeb.fullNode.request('wallet/updatesetting', {
        owner_address: toHex(ownerAddress),
        contract_address: toHex(contractAddress),
        consume_user_resource_percent: userFeePercentage
      }, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
    /**
     * Update energy limit.
     */

  }, {
    key: "updateEnergyLimit",
    value: function updateEnergyLimit() {
      var contractAddress = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var originEnergyLimit = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var ownerAddress = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this.mcashWeb.defaultAddress.hex;
      var callback = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

      if (_utils["default"].isFunction(ownerAddress)) {
        callback = ownerAddress;
        ownerAddress = this.mcashWeb.defaultAddress.hex;
      }

      if (!callback) return this.injectPromise(this.updateEnergyLimit, contractAddress, originEnergyLimit, ownerAddress);
      if (this.validator.notValid([{
        name: 'owner',
        type: 'address',
        value: ownerAddress
      }, {
        name: 'contract',
        type: 'address',
        value: contractAddress
      }, {
        name: 'originEnergyLimit',
        type: 'integer',
        value: originEnergyLimit,
        gte: 0,
        lte: 10000000
      }], callback)) return;
      this.mcashWeb.fullNode.request('wallet/updateenergylimit', {
        owner_address: toHex(ownerAddress),
        contract_address: toHex(contractAddress),
        origin_energy_limit: originEnergyLimit
      }, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
  }, {
    key: "checkPermissions",
    value: function checkPermissions(permissions, type) {
      if (permissions) {
        if (permissions.type !== type || !permissions.permission_name || !_utils["default"].isString(permissions.permission_name) || !_utils["default"].isInteger(permissions.threshold) || permissions.threshold < 1 || !permissions.keys) {
          return false;
        }

        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = permissions.keys[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var key = _step2.value;

            if (!this.mcashWeb.isAddress(key.address) || !_utils["default"].isInteger(key.weight) || key.weight > permissions.threshold || key.weight < 1 || type === 2 && !permissions.operations) {
              return false;
            }
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2["return"] != null) {
              _iterator2["return"]();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }
      }

      return true;
    }
  }, {
    key: "updateAccountPermissions",
    value: function updateAccountPermissions() {
      var ownerAddress = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.mcashWeb.defaultAddress.hex;
      var ownerPermissions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var witnessPermissions = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
      var activesPermissions = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
      var callback = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;

      if (_utils["default"].isFunction(activesPermissions)) {
        callback = activesPermissions;
        activesPermissions = false;
      }

      if (_utils["default"].isFunction(witnessPermissions)) {
        callback = witnessPermissions;
        witnessPermissions = activesPermissions = false;
      }

      if (_utils["default"].isFunction(ownerPermissions)) {
        callback = ownerPermissions;
        ownerPermissions = witnessPermissions = activesPermissions = false;
      }

      if (!callback) return this.injectPromise(this.updateAccountPermissions, ownerAddress, ownerPermissions, witnessPermissions, activesPermissions);
      if (!this.mcashWeb.isAddress(ownerAddress)) return callback('Invalid ownerAddress provided');

      if (!this.checkPermissions(ownerPermissions, 0)) {
        return callback('Invalid ownerPermissions provided');
      }

      if (!this.checkPermissions(witnessPermissions, 1)) {
        return callback('Invalid witnessPermissions provided');
      }

      if (!Array.isArray(activesPermissions)) {
        activesPermissions = [activesPermissions];
      }

      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = activesPermissions[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var activesPermission = _step3.value;

          if (!this.checkPermissions(activesPermission, 2)) {
            return callback('Invalid activesPermissions provided');
          }
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3["return"] != null) {
            _iterator3["return"]();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }

      var data = {
        owner_address: ownerAddress
      };

      if (ownerPermissions) {
        data.owner = ownerPermissions;
      }

      if (witnessPermissions) {
        data.witness = witnessPermissions;
      }

      if (activesPermissions) {
        data.actives = activesPermissions.length === 1 ? activesPermissions[0] : activesPermissions;
      }

      this.mcashWeb.fullNode.request('wallet/accountpermissionupdate', data, 'post').then(function (transaction) {
        return resultManager(transaction, callback);
      })["catch"](function (err) {
        return callback(err);
      });
    }
  }]);
  return TransactionBuilder;
}();

exports["default"] = TransactionBuilder;
//# sourceMappingURL=transactionBuilder.js.map