"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.bin2String = bin2String;
exports.arrayEquals = arrayEquals;
exports.stringToBytes = stringToBytes;
exports.hexChar2byte = hexChar2byte;
exports.isHexChar = isHexChar;
exports.hexStr2byteArray = hexStr2byteArray;
exports.strToDate = strToDate;
exports.isNumber = isNumber;
exports.getStringType = getStringType;
Object.defineProperty(exports, "byte2hexStr", {
  enumerable: true,
  get: function get() {
    return _bytes.byte2hexStr;
  }
});
Object.defineProperty(exports, "bytesToString", {
  enumerable: true,
  get: function get() {
    return _bytes.bytesToString;
  }
});
Object.defineProperty(exports, "hextoString", {
  enumerable: true,
  get: function get() {
    return _bytes.hextoString;
  }
});
Object.defineProperty(exports, "byteArray2hexStr", {
  enumerable: true,
  get: function get() {
    return _bytes.byteArray2hexStr;
  }
});
Object.defineProperty(exports, "base64DecodeFromString", {
  enumerable: true,
  get: function get() {
    return _bytes.base64DecodeFromString;
  }
});
Object.defineProperty(exports, "base64EncodeToString", {
  enumerable: true,
  get: function get() {
    return _bytes.base64EncodeToString;
  }
});

var _bytes = require("./bytes");

function bin2String(array) {
  // TODO Do we need this alias?
  return (0, _bytes.bytesToString)(array);
}

function arrayEquals(array1, array2, strict) {
  if (array1.length != array2.length) return false;
  var i;

  for (i = 0; i < array1.length; i++) {
    if (strict) {
      if (array1[i] != array2[i]) return false;
    } else if (JSON.stringify(array1[i]) != JSON.stringify(array2[i])) return false;
  }

  return true;
}

function stringToBytes(str) {
  if (typeof str !== 'string') throw new Error('The passed string is not a string');
  var bytes = new Array();
  var len;
  var c;
  len = str.length;

  for (var i = 0; i < len; i++) {
    c = str.charCodeAt(i);

    if (c >= 0x010000 && c <= 0x10FFFF) {
      bytes.push(c >> 18 & 0x07 | 0xF0);
      bytes.push(c >> 12 & 0x3F | 0x80);
      bytes.push(c >> 6 & 0x3F | 0x80);
      bytes.push(c & 0x3F | 0x80);
    } else if (c >= 0x000800 && c <= 0x00FFFF) {
      bytes.push(c >> 12 & 0x0F | 0xE0);
      bytes.push(c >> 6 & 0x3F | 0x80);
      bytes.push(c & 0x3F | 0x80);
    } else if (c >= 0x000080 && c <= 0x0007FF) {
      bytes.push(c >> 6 & 0x1F | 0xC0);
      bytes.push(c & 0x3F | 0x80);
    } else bytes.push(c & 0xFF);
  }

  return bytes;
}

function hexChar2byte(c) {
  var d;
  if (c >= 'A' && c <= 'F') d = c.charCodeAt(0) - 'A'.charCodeAt(0) + 10;else if (c >= 'a' && c <= 'f') d = c.charCodeAt(0) - 'a'.charCodeAt(0) + 10;else if (c >= '0' && c <= '9') d = c.charCodeAt(0) - '0'.charCodeAt(0);
  if (typeof d === 'number') return d;else throw new Error('The passed hex char is not a valid hex char');
}

function isHexChar(c) {
  if (c >= 'A' && c <= 'F' || c >= 'a' && c <= 'f' || c >= '0' && c <= '9') {
    return 1;
  }

  return 0;
}

function hexStr2byteArray(str) {
  if (typeof str !== 'string') throw new Error('The passed string is not a string');
  var byteArray = Array();
  var d = 0;
  var j = 0;
  var k = 0;

  for (var i = 0; i < str.length; i++) {
    var c = str.charAt(i);

    if (isHexChar(c)) {
      d <<= 4;
      d += hexChar2byte(c);
      j++;

      if (0 === j % 2) {
        byteArray[k++] = d;
        d = 0;
      }
    } else throw new Error('The passed hex char is not a valid hex string');
  }

  return byteArray;
} //yyyy-MM-DD HH-mm-ss


function strToDate(str) {
  if (!/^\d{4}-\d{2}-\d{2}( \d{2}-\d{2}-\d{2}|)/.test(str)) throw new Error('The passed date string is not valid');
  var tempStrs = str.split(" ");
  var dateStrs = tempStrs[0].split("-");
  var year = parseInt(dateStrs[0], 10);
  var month = parseInt(dateStrs[1], 10) - 1;
  var day = parseInt(dateStrs[2], 10);

  if (tempStrs.length > 1) {
    var timeStrs = tempStrs[1].split("-");
    var hour = parseInt(timeStrs[0], 10);
    var minute = parseInt(timeStrs[1], 10);
    var second = parseInt(timeStrs[2], 10);
    return new Date(year, month, day, hour, minute, second);
  }

  return new Date(year, month, day);
}

function isNumber(c) {
  if (c >= '0' && c <= '9') return 1;
  return 0;
} //return 1: address  --- 20Bytes HexString
//return 2: blockNumber ------ Decimal number
//return 3: assetName ------ String
//return other: error


function getStringType(str) {
  if (null == str) return -1;
  if (typeof str != 'string') return -1;
  if (str.length == 0 || str == "") return -1;
  var i = 0; // TODO Should we return 1 if someone passes a full, 42-chars long address?
  // if (str.length == 42 && /^41/.test(str)) {
  //     for (; i < 40; i++) {
  //         var c = str.charAt(i+2);
  //
  //         if (!isHexChar(c))
  //             break;
  //     }
  // } else

  if (str.length == 40) {
    for (; i < 40; i++) {
      var c = str.charAt(i);
      if (!isHexChar(c)) break;
    }
  }

  if (i == 40) return 1; //40 Hex, Address

  for (i = 0; i < str.length; i++) {
    var c = str.charAt(i);
    if (!isNumber(c)) break;
  }

  if (i == str.length) return 2; // All Decimal number, BlockNumber

  for (i = 0; i < str.length; i++) {
    var c = str.charAt(i);
    if (c > ' ') return 3; // At least one visible character
  }

  return -1;
}
//# sourceMappingURL=code.js.map