"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getBase58CheckAddress = getBase58CheckAddress;
exports.decodeBase58Address = decodeBase58Address;
exports.signTransaction = signTransaction;
exports.arrayToBase64String = arrayToBase64String;
exports.signBytes = signBytes;
exports.getRowBytesFromTransactionBase64 = getRowBytesFromTransactionBase64;
exports.genPriKey = genPriKey;
exports.computeAddress = computeAddress;
exports.getAddressFromPriKey = getAddressFromPriKey;
exports.decode58Check = decode58Check;
exports.isAddressValid = isAddressValid;
exports.getBase58CheckAddressFromPriKeyBase64String = getBase58CheckAddressFromPriKeyBase64String;
exports.getHexStrAddressFromPriKeyBase64String = getHexStrAddressFromPriKeyBase64String;
exports.getAddressFromPriKeyBase64String = getAddressFromPriKeyBase64String;
exports.getPubKeyFromPriKey = getPubKeyFromPriKey;
exports.ECKeySign = ECKeySign;
exports.SHA256 = SHA256;
exports.passwordToAddress = passwordToAddress;
exports.pkToAddress = pkToAddress;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _address = require("./address");

var _code = require("./code");

var _base = require("./base58");

var _bytes = require("./bytes");

var _elliptic = require("elliptic");

var _ethersUtils = require("./ethersUtils");

function getBase58CheckAddress(addressBytes) {
  var hash0 = SHA256(addressBytes);
  var hash1 = SHA256(hash0);
  var checkSum = hash1.slice(0, 4);
  checkSum = addressBytes.concat(checkSum);
  return (0, _base.encode58)(checkSum);
}

function decodeBase58Address(base58Sting) {
  if (typeof base58Sting != 'string') return false;
  if (base58Sting.length <= 4) return false;
  var address = (0, _base.decode58)(base58Sting);
  if (base58Sting.length <= 4) return false;
  var len = address.length;
  var offset = len - 4;
  var checkSum = address.slice(offset);
  address = address.slice(0, offset);
  var hash0 = SHA256(address);
  var hash1 = SHA256(hash0);
  var checkSum1 = hash1.slice(0, 4);

  if (checkSum[0] == checkSum1[0] && checkSum[1] == checkSum1[1] && checkSum[2] == checkSum1[2] && checkSum[3] == checkSum1[3]) {
    return address;
  }

  throw new Error('Invalid address provided');
}

function signTransaction(priKeyBytes, transaction) {
  if (typeof priKeyBytes === 'string') priKeyBytes = (0, _code.hexStr2byteArray)(priKeyBytes);
  var txId = transaction.tx_id;
  var signature = ECKeySign((0, _code.hexStr2byteArray)(txId), priKeyBytes);

  if (Array.isArray(transaction.signature)) {
    if (!transaction.signature.includes(signature)) transaction.signature.push(signature);
  } else transaction.signature = [signature];

  return transaction;
}

function arrayToBase64String(a) {
  return btoa(String.fromCharCode.apply(String, (0, _toConsumableArray2["default"])(a)));
}

function signBytes(privateKey, contents) {
  if (typeof privateKey === 'string') privateKey = (0, _code.hexStr2byteArray)(privateKey);
  var hashBytes = SHA256(contents);
  var signBytes = ECKeySign(hashBytes, privateKey);
  return signBytes;
}

function getRowBytesFromTransactionBase64(base64Data) {
  var bytesDecode = (0, _code.base64DecodeFromString)(base64Data);
  var transaction = proto.protocol.Transaction.deserializeBinary(bytesDecode);
  var raw = transaction.getRawData();
  return raw.serializeBinary();
}

function genPriKey() {
  var ec = new _elliptic.ec('secp256k1');
  var key = ec.genKeyPair();
  var priKey = key.getPrivate();
  var priKeyHex = priKey.toString('hex');

  while (priKeyHex.length < 64) {
    priKeyHex = "0".concat(priKeyHex);
  }

  return (0, _code.hexStr2byteArray)(priKeyHex);
}

function computeAddress(pubBytes) {
  if (pubBytes.length === 65) pubBytes = pubBytes.slice(1);
  var hash = (0, _ethersUtils.keccak256)(pubBytes).toString().substring(2);
  var addressHex = _address.ADDRESS_PREFIX + hash.substring(24);
  return (0, _code.hexStr2byteArray)(addressHex);
}

function getAddressFromPriKey(priKeyBytes) {
  var pubBytes = getPubKeyFromPriKey(priKeyBytes);
  return computeAddress(pubBytes);
}

function decode58Check(addressStr) {
  var decodeCheck = (0, _base.decode58)(addressStr);
  if (decodeCheck.length <= 4) return false;
  var decodeData = decodeCheck.slice(0, decodeCheck.length - 4);
  var hash0 = SHA256(decodeData);
  var hash1 = SHA256(hash0);

  if (hash1[0] === decodeCheck[decodeData.length] && hash1[1] === decodeCheck[decodeData.length + 1] && hash1[2] === decodeCheck[decodeData.length + 2] && hash1[3] === decodeCheck[decodeData.length + 3]) {
    return decodeData;
  }

  return false;
}

function isAddressValid(base58Str) {
  if (typeof base58Str !== 'string') return false;
  if (base58Str.length !== _address.ADDRESS_SIZE) return false;
  var address = (0, _base.decode58)(base58Str);
  if (address.length !== 25) return false;
  if (address[0] !== _address.ADDRESS_PREFIX_BYTE) return false;
  var checkSum = address.slice(21);
  address = address.slice(0, 21);
  var hash0 = SHA256(address);
  var hash1 = SHA256(hash0);
  var checkSum1 = hash1.slice(0, 4);

  if (checkSum[0] == checkSum1[0] && checkSum[1] == checkSum1[1] && checkSum[2] == checkSum1[2] && checkSum[3] == checkSum1[3]) {
    return true;
  }

  return false;
}

function getBase58CheckAddressFromPriKeyBase64String(priKeyBase64String) {
  var priKeyBytes = (0, _code.base64DecodeFromString)(priKeyBase64String);
  var pubBytes = getPubKeyFromPriKey(priKeyBytes);
  var addressBytes = computeAddress(pubBytes);
  return getBase58CheckAddress(addressBytes);
}

function getHexStrAddressFromPriKeyBase64String(priKeyBase64String) {
  var priKeyBytes = (0, _code.base64DecodeFromString)(priKeyBase64String);
  var pubBytes = getPubKeyFromPriKey(priKeyBytes);
  var addressBytes = computeAddress(pubBytes);
  var addressHex = (0, _bytes.byteArray2hexStr)(addressBytes);
  return addressHex;
}

function getAddressFromPriKeyBase64String(priKeyBase64String) {
  var priKeyBytes = (0, _code.base64DecodeFromString)(priKeyBase64String);
  var pubBytes = getPubKeyFromPriKey(priKeyBytes);
  var addressBytes = computeAddress(pubBytes);
  var addressBase64 = (0, _code.base64EncodeToString)(addressBytes);
  return addressBase64;
}

function getPubKeyFromPriKey(priKeyBytes) {
  var ec = new _elliptic.ec('secp256k1');
  var key = ec.keyFromPrivate(priKeyBytes, 'bytes');
  var pubkey = key.getPublic();
  var x = pubkey.x;
  var y = pubkey.y;
  var xHex = x.toString('hex');

  while (xHex.length < 64) {
    xHex = "0".concat(xHex);
  }

  var yHex = y.toString('hex');

  while (yHex.length < 64) {
    yHex = "0".concat(yHex);
  }

  var pubkeyHex = "04".concat(xHex).concat(yHex);
  var pubkeyBytes = (0, _code.hexStr2byteArray)(pubkeyHex);
  return pubkeyBytes;
}

function ECKeySign(hashBytes, priKeyBytes) {
  var ec = new _elliptic.ec('secp256k1');
  var key = ec.keyFromPrivate(priKeyBytes, 'bytes');
  var signature = key.sign(hashBytes);
  var r = signature.r;
  var s = signature.s;
  var id = signature.recoveryParam;
  var rHex = r.toString('hex');

  while (rHex.length < 64) {
    rHex = "0".concat(rHex);
  }

  var sHex = s.toString('hex');

  while (sHex.length < 64) {
    sHex = "0".concat(sHex);
  }

  var idHex = (0, _bytes.byte2hexStr)(id);
  var signHex = rHex + sHex + idHex;
  return signHex;
}

function SHA256(msgBytes) {
  var msgHex = (0, _bytes.byteArray2hexStr)(msgBytes);
  var hashHex = (0, _ethersUtils.sha256)('0x' + msgHex).replace(/^0x/, '');
  return (0, _code.hexStr2byteArray)(hashHex);
}

function passwordToAddress(password) {
  var com_priKeyBytes = (0, _code.base64DecodeFromString)(password);
  var com_addressBytes = getAddressFromPriKey(com_priKeyBytes);
  return getBase58CheckAddress(com_addressBytes);
}

function pkToAddress(privateKey) {
  var com_priKeyBytes = (0, _code.hexStr2byteArray)(privateKey);
  var com_addressBytes = getAddressFromPriKey(com_priKeyBytes);
  return getBase58CheckAddress(com_addressBytes);
}
//# sourceMappingURL=crypto.js.map