"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var accounts = _interopRequireWildcard(require("./accounts"));

var base58 = _interopRequireWildcard(require("./base58"));

var bytes = _interopRequireWildcard(require("./bytes"));

var crypto = _interopRequireWildcard(require("./crypto"));

var code = _interopRequireWildcard(require("./code"));

var abi = _interopRequireWildcard(require("./abi"));

var _validator = _interopRequireDefault(require("validator"));

var _bignumber = _interopRequireDefault(require("bignumber.js"));

var _address = require("utils/address");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var utils = {
  isValidURL: function isValidURL(url) {
    if (typeof url !== 'string') return false;
    return _validator["default"].isURL(url, {
      protocols: ['http', 'https'],
      require_tld: true
    });
  },
  isObject: function isObject(obj) {
    return obj === Object(obj) && Object.prototype.toString.call(obj) !== '[object Array]';
  },
  isArray: function isArray(array) {
    return Array.isArray(array);
  },
  isJson: function isJson(string) {
    try {
      return !!JSON.parse(string);
    } catch (ex) {
      return false;
    }
  },
  isBoolean: function isBoolean(bool) {
    return typeof bool === 'boolean';
  },
  isBigNumber: function isBigNumber(number) {
    return number && (number instanceof _bignumber["default"] || number.constructor && number.constructor.name === 'BigNumber');
  },
  isString: function isString(string) {
    return typeof string === 'string' || string && string.constructor && string.constructor.name === 'String';
  },
  isFunction: function isFunction(obj) {
    return typeof obj === 'function';
  },
  isHex: function isHex(string) {
    return typeof string === 'string' && !isNaN(parseInt(string, 16)) && /^(0x|)[a-fA-F0-9]+$/.test(string);
  },
  isInteger: function isInteger(number) {
    if (number === null) return false;
    return Number.isInteger(Number(number));
  },
  hasProperty: function hasProperty(obj, property) {
    return Object.prototype.hasOwnProperty.call(obj, property);
  },
  hasProperties: function hasProperties(obj) {
    var _this = this;

    for (var _len = arguments.length, properties = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      properties[_key - 1] = arguments[_key];
    }

    return properties.length && !properties.map(function (property) {
      return _this.hasProperty(obj, property);
    }).includes(false);
  },
  injectPromise: function injectPromise(func) {
    for (var _len2 = arguments.length, args = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
      args[_key2 - 1] = arguments[_key2];
    }

    return new Promise(function (resolve, reject) {
      func.apply(void 0, args.concat([function (err, res) {
        if (err) reject(err);else resolve(res);
      }]));
    });
  },
  promiseInjector: function promiseInjector(scope) {
    var _this2 = this;

    return function (func) {
      for (var _len3 = arguments.length, args = new Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
        args[_key3 - 1] = arguments[_key3];
      }

      return _this2.injectPromise.apply(_this2, [func.bind(scope)].concat(args));
    };
  },
  mapEvent: function mapEvent(event) {
    var data = {
      block: event.block_number,
      timestamp: event.block_timestamp,
      contract: event.contract_address,
      name: event.event_name,
      transaction: event.transaction_id,
      result: event.result,
      resourceNode: event.resource_Node || (event._unconfirmed ? 'fullNode' : 'solidityNode')
    };

    if (event._unconfirmed) {
      data.unconfirmed = event._unconfirmed;
    }

    if (event._fingerprint) {
      data.fingerprint = event._fingerprint;
    }

    return data;
  },
  parseEvent: function parseEvent(event, _ref) {
    var abi = _ref.inputs;
    if (!event.result) return event;

    if (this.isObject(event.result)) {
      for (var i = 0; i < abi.length; i++) {
        var obj = abi[i];
        if (obj.type == 'address' && obj.name in event.result) event.result[obj.name] = _address.ADDRESS_PREFIX + event.result[obj.name].substr(2).toLowerCase();
      }
    } else if (this.isArray(event.result)) {
      event.result = event.result.reduce(function (obj, result, index) {
        var _abi$index = abi[index],
            name = _abi$index.name,
            type = _abi$index.type;
        if (type == 'address') result = _address.ADDRESS_PREFIX + result.substr(2).toLowerCase();
        obj[name] = result;
        return obj;
      }, {});
    }

    return event;
  },
  padLeft: function padLeft(input, padding, amount) {
    var res = input.toString();

    while (res.length < amount) {
      res = padding + res;
    }

    return res;
  },
  isNotNullOrUndefined: function isNotNullOrUndefined(val) {
    return val !== null && typeof val !== 'undefined';
  }
};

var _default = _objectSpread({}, utils, {
  code: code,
  accounts: accounts,
  base58: base58,
  bytes: bytes,
  crypto: crypto,
  abi: abi
});

exports["default"] = _default;
//# sourceMappingURL=index.js.map